/*
 * Main.cpp
 *
 */

#include "Scene.h"
#include "MainViewer.h"
#include "PlaneViewer.h"
#include "ViewerPool.h"

#include <QImage>
#include <QtGui>
#include <QApplication>
#include <QStringList>

int main(int argc, char *argv[]) {
	// Read command lines arguments
	QApplication application(argc, argv);
	QStringList arg = application.arguments();

	// Create Splitters
	QSplitter hSplit(Qt::Vertical);
	QSplitter vSplit1(&hSplit);
	QSplitter vSplit2(&hSplit);

	// Create the scene
	Scene scene;
	ViewerPool pool;

	// Instantiate the section viewer
	PlaneViewer section(&scene, &hSplit, 0);

	// Instantiate the main viewers
	MainViewer side(&scene, &pool, &section, 0, &vSplit1, &section);
	MainViewer top(&scene, &pool, &section, 1, &vSplit1, &section);
	MainViewer front(&scene, &pool, &section, 2, &vSplit2, &section);
	MainViewer persp(&scene, &pool, &section, 3, &vSplit2, &section);

	// Add main viewers to the pool
	pool.addViewer(&side);
	pool.addViewer(&top);
	pool.addViewer(&front);
	pool.addViewer(&persp);

	// Set main QSplitter as the main widget
	hSplit.setWindowTitle("3D Wood Viewer");
	section.setWindowTitle("Wood Section Viewer");
	hSplit.show();
	section.show();

	// Run main loop
	return application.exec();
}

