/*
 * PlaneViewer.h
 *
 */

#ifndef PLANEVIEWER_H_
#define PLANEVIEWER_H_

#include <QGLViewer/qglviewer.h>

// Class declaration
namespace qglviewer {
	class Frame;
}
class Scene;

class PlaneViewer: public QGLViewer
{
Q_OBJECT
	typedef qglviewer::Frame Frame;
	typedef qglviewer::Vec Vec;

public:
	PlaneViewer(Scene* s, QWidget* m, QWidget* parent, const QGLWidget* shareWidget = 0);

protected:
	virtual void init();
	virtual void draw();
	virtual void drawWithNames();
	virtual QString helpString() const;
	virtual void setKeyDescriptions();

	virtual void keyPressEvent(QKeyEvent *event);
	virtual void mousePressEvent(QMouseEvent *event);
	virtual void mouseMoveEvent(QMouseEvent *event);
	virtual void postSelection(const QPoint& point);

public slots:
	void updateCamera();

private:
	void loadSection();
	void saveSection();
	void loadParameters();
	const Frame* plane() const;
	Vec convertClickToPlane(const QPoint &pixel) const;
	QPointF projectOnPlane(const Vec &pos) const;
	QPointF toPlane(const QPoint &pixel) const;

private:
	Scene *scene;
	QWidget *mainViewer;
};

#endif /* PLANEVIEWER_H_ */
