/*
 * CellIntersections.h
 *
 */

#ifndef CELLINTERSECTIONS_H_
#define CELLINTERSECTIONS_H_

// Include
#include "Cell.h"
#include <QVector>
#include <QStack>
#include <QPair>

// Struct definitions
class CellIntersections
{
	typedef QPair<Cell *, Cell *> Intersection;

public:
	CellIntersections();

	void push(Cell *a, Cell *b, bool side, double t);
	void minorGather();
	void finalGather();
	int size() const;

	double minParameter;
	QVector<Intersection> interLeft;
	QVector<Intersection> interRight;

private:
	void appendLeft(const Intersection &inter);
	void appendRight(const Intersection &inter);
	void unroll(QStack<QPair<Intersection, bool> > &stack);

	QStack<QPair<Intersection, bool> > firstHalf;
	QStack<QPair<Intersection, bool> > secondHalf;
	QStack<Intersection> tmpLeft;
};

/*****************************************************************************
 CellIntersections inline methods
 *****************************************************************************/

inline CellIntersections::CellIntersections() :
	minParameter(1.0) {
}

inline void CellIntersections::push(Cell *a, Cell *b, bool side, double t) {
	minParameter = std::min(minParameter, t);
	if (t > minParameter)
		secondHalf.push(qMakePair(Intersection(a, b), side));
	else
		firstHalf.push(qMakePair(Intersection(a, b), side));
}

inline void CellIntersections::minorGather() {
	unroll(firstHalf);
	unroll(secondHalf);
}

inline void CellIntersections::finalGather() {
	while (not tmpLeft.isEmpty())
		interLeft.append(tmpLeft.pop());
	Q_ASSERT(interLeft.size() == interRight.size());
}

inline int CellIntersections::size() const {
	return interRight.size();
}

inline void CellIntersections::appendLeft(const Intersection &inter) {
	if (interRight.isEmpty())
		tmpLeft.append(inter);
	else
		interLeft.append(inter);
}

inline void CellIntersections::appendRight(const Intersection &inter) {
	interRight.append(inter);
}

inline void CellIntersections::unroll(QStack<QPair<Intersection, bool> > &stack) {
	while (not stack.isEmpty()) {
		if (stack.top().second)
			appendLeft(stack.pop().first);
		else
			appendRight(stack.pop().first);
	}
}

#endif /* CELLINTERSECTIONS_H_ */
