#ifndef _MESHIMPORTER_H_
#define _MESHIMPORTER_H_

#include "Mesh.h"

#include <QTextStream>
#include <QtGlobal>
#include <QString>
#include <QMap>
#include <QPair>
#include <QVector>
#include <QVector2D>
#include <QVector3D>

class MeshImporter
{
public:
	bool import(const QString &filename, Mesh &mesh);

private:
	struct IndexTriplet
	{
		uint i1, i2, i3;
	};

private:
	void clear();

	bool readData(QTextStream &inputFile, Mesh &mesh);
	void readFace(QTextStream &inputFile, Mesh &mesh);
	void skipLine(QTextStream &inputFile);
	QVector2D readVector2D(QTextStream &inputFile);
	QVector3D readVector3D(QTextStream &inputFile);
	void readTriplet(QTextStream &inputLine, IndexTriplet &triplet);
	uint createVertex(const IndexTriplet &triplet, Mesh &mesh);

private:
	typedef Mesh::Face Face;
	typedef QPair<uint, uint> IndexPair;
	typedef QMap<IndexPair, uint> VertexMap;

private:
	bool mHasTexCoords;
	bool mHasNormals;
	QVector<QVector3D> mPositions;
	QVector<QVector3D> mNormals;
	QVector<QVector3D> mTexCoords;
	QVector<VertexMap> mVertexMap;
};

#endif //_MESHIMPORTER_H_
