/*
 * Scene.h
 *
 */

#ifndef SCENE_H_
#define SCENE_H_

// Include
#include <QString>
#include <QVector>
#include <QObject>

// Class declaration
namespace qglviewer {
	class ManipulatedFrame;
}
class Object;
class WoodSection;
class Parameters;

// Class definition
class Scene: public QObject
{
Q_OBJECT

	typedef qglviewer::ManipulatedFrame ManipulatedFrame;

public:
	Scene();
	virtual ~Scene();

	void draw() const;
	void drawWithNames() const;
	void loadMesh(const QString &fileName);
	void loadSection(const QString &fileName);
	void saveSection(const QString &fileName);
	void loadParameters(const QString &fileName);
	void diffIteration(int i);

	float radius() const;
	WoodSection* woodSection() const;
	ManipulatedFrame* manipulatedFrame() const;
	const Parameters *parameters() const;

public slots:
	void computeRadius();

signals:
	void radiusChanged(float newValue);
	void radiusChanged();
	void sceneChanged();
	void planeModified();

private:
	float sceneRadius;

	Parameters *params;
	ManipulatedFrame* frame;
	WoodSection *currentSection;
	QVector<Object *> objects;
};

// Inline methods
inline float Scene::radius() const {
	return sceneRadius;
}

inline WoodSection* Scene::woodSection() const {
	return currentSection;
}

inline qglviewer::ManipulatedFrame* Scene::manipulatedFrame() const {
	return frame;
}

inline const Parameters* Scene::parameters() const {
	return params;
}

#endif /* SCENE_H_ */
