/*
 * Parameters.cpp
 *
 */

#include "Parameters.h"
#include "Random.h"
#include <QTextStream>
#include <QFile>

Parameters::Parameters() {
	defaultRandomSeed = time(0);
	//defaultRandomSeed = 1313879588ull;
	//defaultRandomSeed = 1311725950ull;
	initialize();
}

Parameters::~Parameters() {
}

/*****************************************************************************
 Default initilization
 *****************************************************************************/

void Parameters::initialize() {
	randomSeed = defaultRandomSeed;
	zeta = rng.uniform(2, 4);
	distanceLimit = 1;
	angularStdev = 0.1;
	lengthStdev = 0.1;
	growthIter = 20;
	stepPerCycle = 5;
	for (int i = 0; i < 4; ++i)
		initialSpeed[i] = 4.0;

	cEarly = QColor::fromRgb(235, 198, 120);
	cLate = QColor::fromRgb(127, 81, 21);
}

/*****************************************************************************
 Load from file
 *****************************************************************************/

void Parameters::load(const QString & fileName) {
	QFile file(fileName);
	if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
		return;
	QTextStream in(&file);

	// Reset the current values to default
	initialize();

	// Load specified values from file
	QString key, arg;
	while (not in.atEnd()) {
		in >> key;
		if (key.startsWith("#")) {
			(void) in.readLine();
			continue;
		}
		if (key == "colorEarly") {
			int r, g, b;
			in >> r >> g >> b;
			cEarly.setRgb(r, g, b);
		} else if (key == "colorLate") {
			int r, g, b;
			in >> r >> g >> b;
			cLate.setRgb(r, g, b);
		} else if (key == "randomSeed")
			in >> randomSeed;
		else if (key == "zeta")
			in >> zeta;
		else if (key == "distanceLimit")
			in >> distanceLimit;
		else if (key == "angularStdev")
			in >> angularStdev;
		else if (key == "lengthStdev")
			in >> lengthStdev;
		else if (key == "growthIter")
			in >> growthIter;
		else if (key == "stepPerCycle")
			in >> stepPerCycle;
		else if (key == "initialSpeed")
			for (int i = 0; i < 4; ++i)
				in >> initialSpeed[i];
	}

	// Setup random seed
	rng.setSeed(randomSeed);
}

void Parameters::showSeed(void) const {
	printf("Random seed: %llu\n", randomSeed);
}
