/*
 * Random.h
 *
 */

#ifndef RANDOM_H_
#define RANDOM_H_

// Include
#include "StdDefinition.h"
#include <ctime>

#ifdef OPENCV
#include <opencv2/core/core.hpp>

// Class definition
class Random
{
public:
	Random(ull seed = time(0)) :
		generator(seed) {
	}

	inline double uniform(double lower, double upper) {
		return generator.uniform(lower, upper);
	}

	inline double normal(double mean, double stdev) {
		return mean + generator.gaussian(stdev);
	}

	inline void setSeed(ull seed) {
		generator.state = seed;
	}

private:
	cv::RNG generator;
};

#else
#include <cstdlib>
#include <cmath>

// Class definition
class Random
{
public:
	Random(ull seed = time(0)) {
		srand(seed);
	}

	inline double uniform(void) {
		return (double) rand() / RAND_MAX;
	}

	inline double uniform(double lower, double upper) {
		return lower + uniform() * (upper - lower);
	}

	inline double normal(void) {
		double u = uniform(), v = uniform();
		return sqrt(-2.0 * log(u)) * cos(2 * Pi * v);
	}

	inline double normal(double mean, double stdev) {
		return mean + normal() * stdev;
	}

	inline void setSeed(ull seed) {
		srand(seed);
	}
};

#endif

// Static random generator
extern Random rng;

#endif /* RANDOM_H_ */
